#!/bin/bash
#
# install-rpm-patches.sh - install patches from a plugin RPM when rm-host 
# isn't running
#
# todo: verify all FXPs have the same VST id

###########################
#	number the patch files
###########################

function NUMBER_FILES()
{
	let COUNT=0
	while read FILE; do
		if [ "$FILE" = ".stub.fxp" ] || [ "$FILE" = ".factory" ] || [ "$FILE" = ".snapshot" ]; then
			continue
		fi

		# already numbered: three digits, a dash, at least one character 
		# then the extension
		# todo: limit to 000 to 128
		TEMP=`echo "$FILE" | grep -E '[0-9][0-9][0-9]-.*\.fxp'`
		if [ "$TEMP" ]; then
			continue
		fi
		
		while [ $COUNT -lt 128 ]; do 
			TEMP=`printf "%03d-" $COUNT`
			if [ ! -e $TEMP* ]; then
				TEMP=`printf "%03d-%s" $COUNT "$FILE"`
				mv "$FILE" "$TEMP"
				let COUNT=COUNT+1
				break
			fi
			let COUNT=COUNT+1
		done

		if [ $COUNT -ge 128 ]; then
			echo Too many files
			exit 1
		fi
	done
	exit 0
}

##################################
# 	main
##################################

# check the directory for validity - ignore wine dir not found warning
BANKS_AND_PATCHES=`winepath -u "c:\\Banks and Patches" 2> /dev/null`
if [ -z "$BANKS_AND_PATCHES" ]; then
	echo install-rpm-patches.sh failed: cannot find Banks and Patches directory
	exit 1
fi
mkdir -p "$BANKS_AND_PATCHES"

# start with a dir
USAGE=
if [ $# -eq 0 ]; then
	USAGE=yes
elif [ $# -eq 1 ]; then
	DIR=$1
elif [ $# -eq 3 ]; then
	if [ $1 = "-root" ]; then
		ROOT=$2
		DIR=$3
	else
		USAGE=yes
	fi
else
	USAGE=yes
fi

if [ "$USAGE" ]; then
	echo "usage: install-rpm-patches.sh -root ROOT DIR"
	echo "Installs a copy of a valid DIR bank into $BANKS_AND_PATCHES,"
	echo "massaging the bank name and patch names as appropriate."
	echo "ROOT defaults to /.  Assumes DIR is of the form"
	echo "$ROOT/c/windows/temp/presets/VENDOR/PLUGIN/BANK.bank or"
	echo "./VENDOR/PLUGIN/BANK.bank."
	echo	
	echo "note: if rm-host is running, use its IPC"
	echo "pipe to install the patches instead."

	exit 1
fi

# check DIR
if [ ! -d "$DIR" ]; then
	echo install-rpm-patches.sh: $DIR is not a directory
	exit 1
fi

# only files
if [ "`find "$DIR" ! -type f`" != "$DIR" ]; then
	echo install-rpm-patches.sh: Files only are permitted
	exit 1
fi

# gotta have a .stub.fxp
if ! ls "$DIR"/.stub.fxp > /dev/null 2>&1; then
	echo install-rpm-patches.sh: No .stub.fxp file found
	exit 1
fi

# .fxp .factory or .snapshot only
TEMP="`find "$DIR" -type f -not -name \*.fxp -not -name .stub.fxp -not -name .factory -not -name .snapshot`"
if [ "$TEMP" ]; then 
	echo install-rpm-patches.sh: .FXP files, .factory or .snapshot only \(\'$TEMP\'\)
	exit 1
fi

# at most 128 fxp files (plus .stub and .factory)
TEMP=`find "$DIR" -type f -name \*.fxp -not -name .stub.fxp | wc -l`
let COUNT="$TEMP"
if [ $COUNT -gt 128 ]; then
	echo install-rpm-patches.sh: At most 128 .FXP files
	exit 1
fi

# extract vendor, plugin and bank name
TEMP=${DIR#"$ROOT/c/windows/Temp/presets/"}
TEMP=${TEMP#"./"}
VENDOR=`echo $TEMP | cut -d / -f 1`
if [ "$VENDOR" = "Multis" ] || [ "$VENDOR" = "Singles" ]; then
	PLUGIN=$VENDOR
	VENDOR=.
else
	PLUGIN=`echo $TEMP | cut -d / -f 2`
fi
BANK=`basename "$DIR"`

if [ ! "$VENDOR" ] || [ ! "$PLUGIN" ] || [ ! "$BANK" ]; then
	echo "install-rpm-patches.sh: Can't find vendor, plugin or bank name"
	exit 1
fi

# work here
WORK=`mktemp -d /tmp/install-rpm-patches.XXXXXX`
cp "$DIR"/* $WORK
cp "$DIR"/.stub.fxp $WORK
if [ -e "$DIR"/.factory ]; then cp "$DIR"/.factory $WORK; fi
if [ -e "$DIR"/.snapshot ]; then cp "$DIR"/.snapshot $WORK; fi

# set patch numbers
(cd $WORK && ls -1 | NUMBER_FILES)

# get highest existing bank number
if [ "$VENDOR" = "." ]; then 
	if [ "$PLUGIN" = "Multis" ]; then
		MSB=000
	elif [ "$PLUGIN" = "Singles" ]; then
		MSB=001
	else
		echo install-rpm-patches.sh: bad vendor name
		exit 1
	fi

	if [ -d "$BANKS_AND_PATCHES/$PLUGIN" ]; then
		LAST_BANK=`find "$BANKS_AND_PATCHES/$PLUGIN" -type d -name "*.bank" \
				-exec basename '{}' \; | sort | tail -1`
	else
		LAST_BANK=
	fi
	if [ "$LAST_BANK" ]; then
		LSB=`echo $LAST_BANK | cut -d \- -f 2`
		if [ "$LSB" \< "127" ] ; then
			let ITEMP=10#$LSB+1
			LSB=`printf "%03d" $ITEMP`
		else
			echo install-rpm-patches.sh: ran out of MSB/LSB values
			exit 1
		fi
	else
		LSB=000
	fi
else 
	# get highest existing bank number > 002-000
	LAST_BANK=`/usr/bin/find "$BANKS_AND_PATCHES" -type d -name "*.bank" \
			-exec basename '{}' \; | sort | tail -1`
	if [ "$LAST_BANK" \< "002-000" ]; then
		MSB=002
		LSB=000
	else
		MSB=`echo $LAST_BANK | cut -d \- -f 1`
		LSB=`echo $LAST_BANK | cut -d \- -f 2`
		if [ "$LSB" \< "127" ] ; then
			let ITEMP=10#$LSB+1
			LSB=`printf "%03d" $ITEMP`
		elif [ "$MSB" \< 127 ]; then
			LSB=000
			let ITEMP=10#$MSB+1
			MSB=`printf "%03d" $ITEMP`
		else
			echo install-rpm-patches.sh: ran out of MSB/LSB values
			exit 1
		fi
	fi
fi
PLUGIN_DIR="$BANKS_AND_PATCHES/$VENDOR/$PLUGIN"
BANK_DIR=$PLUGIN_DIR"/"$MSB"-"$LSB"-"$BANK

# move the bank folder over
mkdir -p "$PLUGIN_DIR"
mv "$WORK" "$BANK_DIR"

# correct permissions
chmod 0755 "$BANK_DIR"
chmod 0644 "$BANK_DIR"/*
chmod 0644 "$BANK_DIR"/.stub.fxp
if [ -e "$BANK_DIR/.factory" ]; then
	chmod 0644 "$BANK_DIR"/.factory
fi
if [ -e "$BANK_DIR/.snapshot" ]; then
	chmod 0644 "$BANK_DIR"/.snapshot
fi
