#!/bin/bash
#
# receptor-recovery.sh -check-button
# 
# Runs receptor's recovery mode. If -check-button, only start
# if the SETUP button is down. Returns 0 if all went well, or
# 1 if the system is being stopped.
#

###############################################################
#
#							CONSTANTS
#
###############################################################

# INPUT: read from the front panel
NEXT_PAGE=nextPage
PREV_PAGE=prevPage
NEXT_VALUE=nextValue
PREV_VALUE=prevValue
APPLY_VALUE=applyValue
INIT_RESPONSE=init
SETUP_DOWN=setupDown
TIMED_OUT=timedOut
# set this to how many seconds to wait for bytes
TIMED_INPUT=

# PAGE/VALUE: page and value selections
declare -i RECOVERY_PAGE=0
declare -i RECOVERY_START=0
declare -i RECOVERY_START_DEFAULT=1
declare -i RECOVERY_REBOOT=2
declare -i RECOVERY_SHUTDOWN=3

declare -i NETWORK_PAGE=1
declare -i NETWORK_AUTO=0
declare -i NETWORK_CROSSOVER=1

declare -i ADDRESS_PAGE=2
declare -i ADDRESS_VALUE=0

declare -i TOOLS_PAGE=3
declare -i TOOLS_FIND=0
declare -i TOOLS_DISK=1
declare -i TOOLS_FIRST=2
# subsequent values are installer rpms

# geber device byte codes
declare -i GEBER_INIT=1
declare -i GEBER_PAGE_KNOB=0x34
declare -i GEBER_VALUE_KNOB=0x35
declare -i GEBER_APPLY_VALUE=0x4C
declare -i GEBER_SETUP_BUTTON=0x47
# frames a geber message
declare -i GEBER_START=0xFE
declare -i GEBER_END=0xFF
# max width for the lcd
declare -i MAX_LCD_LENGTH=24
# command to init the display - and get a record of down buttons
GEBER_INIT_COMMAND="\xfe\x2\x1\x1\x2\xff"

# front panel display app
FRONT_PANEL_APP=/usr/bin/front-panel
NORMAL_EXIT=normalExit
SHUTDOWN_EXIT=shutdownExit

###############################################################
#
#							VARIABLES
#
###############################################################

# input read from front-panel - see get-input
INPUT=
# byte read from front panel - see get-byte
declare -i BYTE=0
# the page and value we are on
declare -i PAGE=0
declare -i VALUE=0
# text to print on front-panel
LINE1=
LINE2=
# array of available installers
declare -a INSTALLER_FILES

# network status and possible values
NETWORK_STATUS=
AUTO_DHCP=auto
CROSSOVER=crossover
OFF=off

# set if networking is in crossover mode
CROSSOVER_IS_UP=

###############################################################
#
#							HELPER FUNCTIONS
#
###############################################################

# apply PAGE VALUE
# apply the setting VALUE for PAGE
#
function apply()
{
	case $PAGE in
		$RECOVERY_PAGE ) 
			case $VALUE in 
				$RECOVERY_START ) 
					$FRONT_PANEL_APP -line2 "Restarting...";
					EXIT=$NORMAL_EXIT
					;;
				$RECOVERY_START_DEFAULT ) 
					# clear away the default files
					rm -f /var/cache/receptor/receptor-plugin-cache.xml
					rm -f /var/cache/receptor/receptor-banks-cache.xml
					rm -f /var/cache/receptor/receptor-auto-save
					$FRONT_PANEL_APP -line2 "Restarting default...";
					EXIT=$NORMAL_EXIT
					;;
				$RECOVERY_REBOOT ) 
					shutdown -r now;
					$FRONT_PANEL_APP -line2 "Rebooting...";
					EXIT=$SHUTDOWN_EXIT
					;;
				$RECOVERY_SHUTDOWN )
					shutdown -h now;
					$FRONT_PANEL_APP -line2 "Shutting down...";
					EXIT=$SHUTDOWN_EXIT
					;;
			esac
			;;
		$NETWORK_PAGE)
			case $VALUE in 
				$NETWORK_AUTO) 
					case $NETWORK_STATUS in
						$AUTO_DHCP) restart-networking;;
						$CROSSOVER) 
							/sbin/service dhcpd stop 
							system-interface tcpip set dhcp
							restart-networking
							;;
						$OFF) 
							# make sure dhcp is on
							system-interface tcpip set dhcp
							start-networking
							;;
					esac
					# keep results on screen for a coupla secs
					sleep 5
					;;
				$NETWORK_CROSSOVER) 
					case $NETWORK_STATUS in
						$AUTO_DHCP)
							system-interface tcpip set crossover
							start-crossover
							;;
						$CROSSOVER) restart-networking;; 
						$OFF) 
							system-interface tcpip set crossover
							start-crossover
							;;
					esac
					# keep results on screen for a coupla secs
					sleep 5
					;;
			esac
			get-network-status
			;;
		$TOOLS_PAGE )
			case $VALUE in
				$TOOLS_FIND)
					get-installer-files
					# show the first
					if [ ${#INSTALLER_FILES[@]} -ne 0 ]; then
						VALUE=$TOOLS_FIRST
					fi
					;;
				$TOOLS_DISK)
					touch /forcefsck
					front-panel -line1 "Checking Disk --- Takes" -line2 "at least 10 mins..."
					shutdown -r now
					EXIT=$SHUTDOWN_EXIT
					;;
				*)
					let I=$VALUE-$TOOLS_FIRST; 
					if rpm -i "/c/Drop\ Installers\ Here/${INSTALLER_FILES[$I]}"; then 
						front-panel -line2 "Install: OK"
					else
						front-panel -line2 "Install: Failed"
					fi
					# hold the display for a few seconds 
					sleep 5
					;;
			esac
			;;
	esac
}

# 
# change-page INPUT
# change to the next or previous page
#
function change-page()
{
	if [ "$1" = "$NEXT_PAGE" ]; then 
		if [ "$PAGE" != "$TOOLS_PAGE" ]; then
			let PAGE=$PAGE+1
		fi
	else
		if [ "$PAGE" != "0" ]; then
			let PAGE=$PAGE-1
		fi
	fi

	# fix VALUE
	VALUE=0
}

# 
# change-value INPUT
# change to the next or previous value
#
function change-value()
{
	local MAX_VALUE
	case $PAGE in
		$RECOVERY_PAGE ) MAX_VALUE=$RECOVERY_SHUTDOWN;;
		$NETWORK_PAGE ) MAX_VALUE=$NETWORK_CROSSOVER;;
		$ADDRESS_PAGE ) MAX_VALUE=$ADDRESS_VALUE;;
		$TOOLS_PAGE ) MAX_VALUE=$((${#INSTALLER_FILES[@]} + $TOOLS_FIRST - 1));
	esac

	if [ "$1" = "$NEXT_VALUE" ]; then 
		if [ "$VALUE" != "$MAX_VALUE" ]; then
			let VALUE=$VALUE+1
		fi
	else
		if [ "$VALUE" != "0" ]; then
			let VALUE=$VALUE-1
		fi
	fi
}

# 
# get-byte
# read the global BYTE from the front panel device
#
function get-byte() {
	# read cannot apparently read directly into an integer var
	if [ "$TIMED_INPUT" ]; then
		REPLY=
		read -u3 -s -n1 -t 1
	else
		read -u3 -s -n1
	fi
	let BYTE=16#`echo -n $REPLY | od -t x1 -A n | tr -d ' '`
}

#
# get-installer-files
# Set INSTALLER_FILES by reading what's in /c/Drop Installers Here
#
get-installer-files() {
	unset INSTALLER_FILES
	local INDEX=0
	for INSTALLER in /c/Drop\ Installers\ Here/*.rpm /c/Drop\ Installers\ Here/*.mrpk; do
		INSTALLER_FILES[$INDEX]=`basename "$INSTALLER"`
		let INDEX=$INDEX+1
	done
}

# 
# get-input
# Set INPUT by reading from the front-panel
#
function get-input()
{
	INPUT=
	# wait for a full command
	while true; do
		# wait for command start
		get-byte
		if [ "$TIMED_INPUT" ] && [ $BYTE -eq 0 ]; then
			INPUT=$TIMED_OUT
			break
		elif [ $BYTE -ne $GEBER_START ]; then
			continue
		fi

		# read the payload count
		get-byte
		if [ $BYTE -eq $GEBER_END ] || [ $BYTE -eq 0 ]; then
			# unexpected end or invalid count
			continue
		fi
		let COUNT=$BYTE
		let SUM=$BYTE

		# read the payload itself
		declare -a DATA
		for (( I = 0; I < $COUNT; I++ )); do
			get-byte
			if [ $BYTE -eq $GEBER_END ]; then
				# unexpected end
				break
			else
				DATA[$I]=$BYTE
				SUM=$(($SUM ^ $BYTE))
			fi
		done			
		if [ $BYTE -eq $GEBER_END ]; then
			# unexpected end
			continue
		fi

		# check sum
		get-byte
		if [ $BYTE -eq $GEBER_END ]; then
			# unexpected end
			continue
		elif [ $BYTE -ne $SUM ]; then
			# bad checksum
			continue
		fi
		
		# terminator
		get-byte
		if [ $BYTE -ne $GEBER_END ]; then
			# expecting terminator
			continue
		fi

		case ${DATA[0]} in
			$GEBER_PAGE_KNOB ) 
				if [ ${#DATA[@]} -eq 2 ]; then
					# todo: if knobs reversed test "if [ $BYTE -lt 64 ]; then"
					if [ ${DATA[1]} -ge 64 ]; then	
						INPUT=$NEXT_PAGE
					else
						INPUT=$PREV_PAGE
					fi
				fi
				;;
			$GEBER_VALUE_KNOB )
				if [ ${#DATA[@]} -eq 2 ]; then
					# todo: if knobs reversed test "if [ $BYTE -lt 64 ]; then"
					if [ ${DATA[1]} -ge 64 ]; then	
						INPUT=$NEXT_VALUE
					else
						INPUT=$PREV_VALUE
					fi
				fi
				;;
			$GEBER_APPLY_VALUE )
				if [ ${#DATA[@]} -eq 2 ]; then
					# check for up/down
					if [ $((${DATA[1]} & 3)) -eq 2 ]; then	
						INPUT=$APPLY_VALUE
					fi
				fi
				;;
			$GEBER_INIT )
				if [ ${#DATA[@]} -eq 2 ] && [ ${DATA[1]} -eq 1 ]; then	
						INPUT=$INIT_RESPONSE
				fi
				;;
			$GEBER_SETUP_BUTTON )
				if [ ${#DATA[@]} -eq 2 ] && [ $((${DATA[1]} & 3)) -eq 1 ]; then	
					INPUT=$SETUP_DOWN
				fi
				;;
		esac

		if [ "$INPUT" ]; then
			break
		fi
	done
}

#
# TEXT get-line1 PAGE
# Set LINE1 to text for PAGE
#
function get-line1()
{
	LINE1=
	case $1 in
		$RECOVERY_PAGE ) LINE1="Receptor Recovery      >";;
		$NETWORK_PAGE )  LINE1="Receptor Networking   <>";;
		$ADDRESS_PAGE )  LINE1="Receptor IP Address   <>";;
		$TOOLS_PAGE )    LINE1="Receptor Tools        <";;
	esac
}

#
# get-line2 PAGE VALUE
# Set LINE2 to text for PAGE and VALUE
#
function get-line2()
{
	LINE2=
	case $1 in
		$RECOVERY_PAGE ) 
			case $2 in 
				$RECOVERY_START ) LINE2="Restart?";;
				$RECOVERY_START_DEFAULT ) LINE2="Reset and restart?";;
				$RECOVERY_REBOOT ) LINE2="Reboot?";;
				$RECOVERY_SHUTDOWN ) LINE2="Shutdown?";;
			esac ;;
		$NETWORK_PAGE )
			case $2 in 
				$NETWORK_AUTO)
					if [ "$NETWORK_STATUS" == "$AUTO_DHCP" ]; then
						LINE2="Auto (DHCP) - restart?"
					else
						LINE2="Start Auto (DHCP)?"
					fi
					;;
				$NETWORK_CROSSOVER)
					if [ "$NETWORK_STATUS" == "$CROSSOVER" ]; then
						LINE2="Crossover - restart?"
					else
						LINE2="Start crossover?"
					fi
					;;
			esac ;;
		$ADDRESS_PAGE)
			LINE2=`/sbin/ifconfig eth0 | sed -n 's/.*inet addr:\(.*\) B.*/\1/p'`;
			if [ ! "$LINE2" ]; then
				LINE2="(none)"
			fi
			;;
		$TOOLS_PAGE )
			case $2 in 
				$TOOLS_FIND ) LINE2="Find installers?";;
				$TOOLS_DISK ) LINE2="Check disk (10 mins)?";;
				* ) LINE2="Run "${INSTALLER_FILES[$(($2-$TOOLS_FIRST))]}"?";;
			esac ;;
	esac

	# limit to MAX_LCD_LENGTH characters
	if [ ${#LINE2} -gt $MAX_LCD_LENGTH ]; then
		let LEN=$MAX_LCD_LENGTH-4
		LINE2=${LINE2:0:$LEN}"...?"
	fi
}

#
# get-network-status
# set the NETWORK_STATUS global
#
get-network-status() {
	# see that eth0 is in the Configured section
	if service network status | grep -A1 "Configured" | tail -1 | grep -q "eth0"; then 
		if service dhcpd status | grep -q "is running"; then 
			NETWORK_STATUS=$CROSSOVER
		elif grep -q BOOTPROTO=dhcp /etc/sysconfig/network-scripts/ifcfg-eth0; then
			NETWORK_STATUS=$AUTO_DHCP
		else
			echo todo: support MANUAL mode
			NETWORK_STATUS=$OFF 
		fi
	else
		NETWORK_STATUS=$OFF
	fi
}

#
# restart-networking
# restart all the network services
#
restart-networking() {
	$FRONT_PANEL_APP -line2 "Restarting..."
	
	# stop ancillary services first
	for SERVICE in sshd smb rendezvous atalk; do
		if service $SERVICE status | grep -q "is running"; then 
			/sbin/service $SERVICE stop
		fi
	done

	# restart networking - require that eth0 be configured to continue
	SERVICE=network
	if [ "$NETWORK_STATUS" != "$OFF" ]; then
		/sbin/service $SERVICE restart
	else
		/sbin/service $SERVICE start
	fi
	get-network-status
	if [ "$NETWORK_STATUS" == "$OFF" ]; then
		$FRONT_PANEL_APP -line2 "Failed (tcpip)"
		return 1
	fi

	# restart the ancillaries
	# note: 'service smb' complains of a broken pipe but it works anwyay...
	for SERVICE in sshd smb rendezvous atalk; do
		/sbin/service $SERVICE start
		if ! service $SERVICE status | grep -q "is running"; then 
			$FRONT_PANEL_APP -line2 "Failed ($SERVICE)"
			return 1
		fi
	done

	$FRONT_PANEL_APP -line2 "Restart: OK"
}

#
# start-networking
# start or restart all the network services
#
start-networking() {
	$FRONT_PANEL_APP -line2 "Starting..."

	# require that eth0 be configured to continue
	SERVICE=network
	if [ "$NETWORK_STATUS" != "$OFF" ]; then
		/sbin/service $SERVICE restart
	else
		/sbin/service $SERVICE start
	fi
	get-network-status
	if [ "$NETWORK_STATUS" == "$OFF" ]; then
		$FRONT_PANEL_APP -line2 "Failed (tcpip)"
		return 1
	fi

	# note: 'service smb' complains of a broken pipe but it works anwyay...
	for SERVICE in sshd smb rendezvous atalk; do
		if service $SERVICE status | grep -q "is running"; then 
			/sbin/service $SERVICE restart
		else
			/sbin/service $SERVICE start
		fi
		if ! service $SERVICE status | grep -q "is running"; then 
			$FRONT_PANEL_APP -line2 "Failed ($SERVICE)"
			return 1
		fi
	done

	$FRONT_PANEL_APP -line2 "Start: OK"
}

#
# start-crossover
# start or restart networking in crossover mode
#
start-crossover() {
	$FRONT_PANEL_APP -line2 "Crossover..."
	
	# stop ancillary services first
	for SERVICE in sshd smb rendezvous atalk; do
		if service $SERVICE status | grep -q "is running"; then 
			/sbin/service $SERVICE stop
		fi
	done

	# restart networking with crossover settings
	if [ "$NETWORK_STATUS" != "$OFF" ]; then
		/sbin/service network restart
	else
		/sbin/service network start
	fi

	# start the dhcpd
	/sbin/service dhcpd start
	if ! service dhcpd status | grep -q "is running"; then 
		$FRONT_PANEL_APP -line2 "Failed (dhcpd)"
		return 1
	fi

	get-network-status
	if [ "$NETWORK_STATUS" != "$CROSSOVER" ]; then
		$FRONT_PANEL_APP -line2 "Failed (tcpip)"
		return 1
	fi

	# restart the ancillaries
	# note: 'service smb' complains of a broken pipe but it works anwyay...
	for SERVICE in sshd smb rendezvous atalk; do
		/sbin/service $SERVICE start
		if ! service $SERVICE status | grep -q "is running"; then 
			$FRONT_PANEL_APP -line2 "Failed ($SERVICE)"
			return 1
		fi
	done

	$FRONT_PANEL_APP -line2 "Crossover: OK"
}

###############################################################
#
#							MAIN
#
###############################################################

# Identify which port the geber device is on
PORT=/dev/panel

# open the front panel
exec 3<> $PORT

# init the display
printf $GEBER_INIT_COMMAND >&3

# start only if SETUP is down
if [ $# -eq 1 ] && [ "$1" == "-check-button" ]; then
	RUN=
	get-input
	if [ $INPUT == $INIT_RESPONSE ]; then
		$FRONT_PANEL_APP -line1 "Press SETUP to enter" -line2 "Receptor Startup Mode..."

		# get down button states 
		TIMED_INPUT=1
		get-input
		if [ $INPUT == $SETUP_DOWN ]; then
			TIMED_INPUT=0
			RUN=1
		fi
	fi
	if [ -z "$RUN" ]; then
		exec 3>&-
		exit 0
	fi
fi

# check on network status
get-network-status

EXIT=
while [ -z "$EXIT" ]; do
	# update the display
	get-line1 $PAGE
	get-line2 $PAGE $VALUE
	$FRONT_PANEL_APP -line1 "$LINE1" -line2 "$LINE2"

	get-input
	case $INPUT in
		$NEXT_PAGE | $PREV_PAGE ) change-page $INPUT;;
		$NEXT_VALUE | $PREV_VALUE ) change-value $INPUT;;
		$APPLY_VALUE ) apply $PAGE $VALUE;;
	esac
done

# close
exec 3>&-

# shutting down
if [ "$EXIT" != "$NORMAL_EXIT" ]; then
	exit 1
fi
exit 0
